// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef TREEVIEW_H
#define TREEVIEW_H

#include "mtp_data_types.h"

#include <gtkmm/treeview.h>
#include <gtkmm/treestore.h>
#include <gtkmm/treerowreference.h>

const double EXPANDER_WIDTH = 17;

class TreeViewBase : public Gtk::TreeView
{
	protected:
		class ModelColumns : public Gtk::TreeModel::ColumnRecord
		{
			public:
				ModelColumns() {add(m_Data);}

				Gtk::TreeModelColumn< Glib::RefPtr<mtp_data_t> > m_Data;
		};

		TreeViewBase(const Glib::ustring& column_title);

		std::vector<Gtk::TreeRowReference> ConvertPathsToRowRefs(
				const std::vector<Gtk::TreePath>& paths);
		std::vector<Gtk::TreeRowReference> ConvertItersToRowRefs(
				const Gtk::TreeNodeChildren& iters);

		Glib::RefPtr<Gtk::TreeStore> m_Store;
		ModelColumns m_StoreColumns;
		Gtk::TreeViewColumn m_Column;

	private:
		void on_row_activated(const Gtk::TreePath& path, Gtk::TreeViewColumn* column);
		bool SearchEqualFunc(const Glib::RefPtr<Gtk::TreeModel>& model,
				int column, const Glib::ustring& key, const Gtk::TreeIter& iter);
		bool QueryTooltip(int x, int y, bool keyboard_tooltip,
				const Glib::RefPtr<Gtk::Tooltip>& tooltip);
};

class DNDTreeViewBase : public TreeViewBase
{
	protected:
		DNDTreeViewBase(const Glib::ustring& column_title);

		double m_DragX;
		double m_DragY;

	private:
		bool on_button_release_event(GdkEventButton* event);
		bool on_motion_notify_event(GdkEventMotion* event);
		void on_drag_data_get(const Glib::RefPtr<Gdk::DragContext>& context,
				Gtk::SelectionData& selection_data, guint info, guint time);
};

#endif // TREEVIEW_H
